@@ -1,6 +1,6 @@
 !***********************************************************************
 ! ISPACK FORTRAN SUBROUTINE LIBRARY FOR SCIENTIFIC COMPUTING
-! Copyright (C) 1998--2021 Keiichi Ishioka <ishioka@gfd-dennou.org>
+! Copyright (C) 1998--2022 Keiichi Ishioka <ishioka@gfd-dennou.org>
 !
 ! This library is free software; you can redistribute it and/or
 ! modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,17 @@ SUBROUTINE LXGPRM(JM,JV,JR)
         JV=JM/2
         JR=1
      END IF
+  ELSE IF(ICPU.EQ.100) THEN ! SX-Aurora
+     IF(MOD(JM,512).EQ.0) THEN
+        JV=256
+        JR=JM/2/JV
+        DO WHILE(MOD(JR,2).EQ.0.AND.JR.GT.64)
+           JR=JR/2
+        END DO
+     ELSE
+        JV=JM/2
+        JR=1
+     END IF
   ELSE
      JV=JM/2
      JR=1
