@@ -1,6 +1,6 @@
 !***********************************************************************
 ! ISPACK FORTRAN SUBROUTINE LIBRARY FOR SCIENTIFIC COMPUTING
-! Copyright (C) 1998--2021 Keiichi Ishioka <ishioka@gfd-dennou.org>
+! Copyright (C) 1998--2022 Keiichi Ishioka <ishioka@gfd-dennou.org>
 !
 ! This library is free software; you can redistribute it and/or
 ! modify it under the terms of the GNU Lesser General Public
@@ -26,35 +26,20 @@ SUBROUTINE SXTG2S(MM,NM,NN,IM,JM,S,G,IT,T,P,R,JC,W,IPOW)
   REAL(8) :: T(IM*3/2),P(JM/2,2*MM+5),R(*)
   INTEGER(8) :: IT(IM/2),JC(*)
   REAL(8) :: W(JM*IM)
-  TYPE(C_PTR),DIMENSION(:),ALLOCATABLE :: WP(:)
+  INTEGER(8) :: NP  
+  TYPE(C_PTR) :: WP
   REAL(8),DIMENSION(:),POINTER:: WORK
-  INTEGER(8) :: NPMAX
-  INTEGER :: IP,NP
-  LOGICAL,DIMENSION(:),ALLOCATABLE :: LP(:)
-  TYPE(C_PTR),DIMENSION(:),ALLOCATABLE :: WSP(:)
+  TYPE(C_PTR) :: WSP
   REAL(8),DIMENSION(:),POINTER:: WS(:)
 
-!$    INTEGER :: omp_get_thread_num  
-
   CALL SXGPRM(JM,JV,JR)  
-
-  CALL MXGOMP(NPMAX)  
-
-  NP=MIN(JM/JV,NPMAX)
-  ALLOCATE(WP(0:NP-1))
-  ALLOCATE(LP(0:NP-1))
-  LP=.FALSE.
-  IP=0
+  CALL MXGOMP(NP)  
   
-!$omp parallel private(IP,WORK) num_threads(NP)
-!$omp do schedule(dynamic)
+!$omp parallel private(WP,WORK) num_threads(NP)
+  CALL MXALLC(WP,JV*IM)
+  CALL C_F_POINTER(WP,WORK,[JV*IM])
+  !$omp do schedule(dynamic)
   DO JD=1,JM/JV
-!$      IP=omp_get_thread_num()
-     IF(.NOT.LP(IP)) THEN
-        CALL MXALLC(WP(IP),JV*IM)
-        CALL C_F_POINTER(WP(IP),WORK,[JV*IM])
-        LP(IP)=.TRUE.
-     END IF
      IF(JV.EQ.4) THEN
         CALL SXQTF2(IM,G(1+JV*IM*(JD-1)),WORK)             
      ELSE IF(JV.EQ.8) THEN     
@@ -71,37 +56,17 @@ SUBROUTINE SXTG2S(MM,NM,NN,IM,JM,S,G,IT,T,P,R,JC,W,IPOW)
         CALL SXLTF1(JV,JM/JV,MM,IM,WORK,W(1+2*JV*(JD-1)))
      END IF
   END DO
-!$omp end do
+  !$omp end do
+  CALL MXFREE(WP)    
 !$omp end parallel
 
-  DO IP=0,NP-1
-     IF(LP(IP)) THEN
-        CALL MXFREE(WP(IP))
-     END IF
-  END DO
-  
-  DEALLOCATE(WP)
-  DEALLOCATE(LP)  
-  
-
-  NP=MIN(MM+1,NPMAX)
-  ALLOCATE(WP(0:NP-1))
-  ALLOCATE(WSP(0:NP-1))  
-  ALLOCATE(LP(0:NP-1))
-  LP=.FALSE.
-  IP=0
-
-!$omp parallel private(NS,IP,IJ,IE,WORK,WS) num_threads(NP)
-!$omp do schedule(dynamic)
+!$omp parallel private(NS,WP,WSP,IJ,IE,WORK,WS) num_threads(NP)
+  CALL MXALLC(WP,JV*7*JR)
+  CALL C_F_POINTER(WP,WORK,[JV*7*JR])
+  CALL MXALLC(WSP,2*(NM+1))
+  CALL C_F_POINTER(WSP,WS,[2*(NM+1)])
+  !$omp do schedule(dynamic)
   DO M=0,MM
-     !$      IP=omp_get_thread_num()
-     IF(.NOT.LP(IP)) THEN
-        CALL MXALLC(WP(IP),JV*7*JR)
-        CALL C_F_POINTER(WP(IP),WORK,[JV*7*JR])
-        CALL MXALLC(WSP(IP),2*(NM+1))
-        CALL C_F_POINTER(WSP(IP),WS,[2*(NM+1)])
-        LP(IP)=.TRUE.
-     END IF
      IF(M.EQ.0) THEN
         CALL LXSGZS(NM,NN,JM,JV,JR,S,W,P,R,WORK,WS,IPOW,0_8)
      ELSE
@@ -112,19 +77,9 @@ SUBROUTINE SXTG2S(MM,NM,NN,IM,JM,S,G,IT,T,P,R,JC,W,IPOW)
         &        R(IE),JC(IJ),WORK,WS,IPOW,0_8)
      END IF
   END DO
-!$omp end do
+  !$omp end do
+  CALL MXFREE(WP)
+  CALL MXFREE(WSP)
 !$omp end parallel
-
-  DO IP=0,NP-1
-     IF(LP(IP)) THEN
-        CALL MXFREE(WP(IP))
-        CALL MXFREE(WSP(IP))        
-     END IF
-  END DO
-
-  DEALLOCATE(WSP)  
-  DEALLOCATE(WP)
-  DEALLOCATE(LP)  
-
   
 END SUBROUTINE SXTG2S
