@@ -1,6 +1,6 @@
 !***********************************************************************
 ! ISPACK FORTRAN SUBROUTINE LIBRARY FOR SCIENTIFIC COMPUTING
-! Copyright (C) 1998--2021 Keiichi Ishioka <ishioka@gfd-dennou.org>
+! Copyright (C) 1998--2022 Keiichi Ishioka <ishioka@gfd-dennou.org>
 !
 ! This library is free software; you can redistribute it and/or
 ! modify it under the terms of the GNU Lesser General Public
@@ -27,47 +27,28 @@ SUBROUTINE SXTS2V(MM,NM,NN,IM,JM,S1,S2,G1,G2,IT,T,P,R,JC,W,IPOW)
   REAL(8) :: T(IM*3/2),P(JM/2,2*MM+5),R(*)
   INTEGER(8) :: IT(IM/2),JC(*)
   REAL(8) :: W(JM*IM,2)
-  TYPE(C_PTR),DIMENSION(:),ALLOCATABLE :: WP(:)
+  INTEGER(8) :: NP  
+  TYPE(C_PTR) :: WP
   REAL(8),DIMENSION(:),POINTER:: WORK
-  INTEGER(8) :: NPMAX
-  INTEGER :: IP,NP
-  LOGICAL,DIMENSION(:),ALLOCATABLE :: LP(:)
-  TYPE(C_PTR),DIMENSION(:),ALLOCATABLE :: WSP(:)
+  TYPE(C_PTR) :: WSP
   REAL(8),DIMENSION(:),POINTER:: WS(:)
 
-!$    INTEGER :: omp_get_thread_num  
-  
   CALL SXGPRM(JM,JV,JR)
   IF(MOD(JR,2).EQ.0) JR=JR/2
 
-  CALL MXGOMP(NPMAX)
+  CALL MXGOMP(NP)
 
-  NP=MIN(MM+1,NPMAX)
-  IP=0
-  ALLOCATE(WP(0:NP-1))
-  ALLOCATE(WSP(0:NP-1))  
-  ALLOCATE(LP(0:NP-1))
-  LP=.FALSE.
-  IP=0
-
-!$omp parallel private(NS,IP,IJ,IE,WORK,WS) num_threads(NP)
-!$omp do schedule(dynamic)
+!$omp parallel private(NS,WP,WSP,IJ,IE,WORK,WS) num_threads(NP)
+  CALL MXALLC(WP,JV*11*JR)
+  CALL C_F_POINTER(WP,WORK,[JV*11*JR])
+  CALL MXALLC(WSP,4*(NM+1))
+  CALL C_F_POINTER(WSP,WS,[4*(NM+1)])
+  !$omp do schedule(dynamic)
   DO M=0,MM
-     !$      IP=omp_get_thread_num()
-     IF(.NOT.LP(IP)) THEN
-        CALL MXALLC(WP(IP),JV*11*JR)
-        CALL C_F_POINTER(WP(IP),WORK,[JV*11*JR])
-        CALL MXALLC(WSP(IP),4*(NM+1))
-        CALL C_F_POINTER(WSP(IP),WS,[4*(NM+1)])
-        LP(IP)=.TRUE.
-     END IF
      IF(M.EQ.0) THEN
         CALL LXSSZV(NM,NN,JM,JV,JR,S1,S2,W(1,1),W(1,2),P,R,WORK,WS,IPOW,0_8)
      ELSE
         NS=1+NN+1+(M-1)*(NN+NN+2-M)
-!        IE=(M*(2*NM-M)+1)/4*3+M*(2*NM-M+1)/2+M+1 ! R礭ƤΤ
-!        IJ=(M-1)*(2*NM-M)/16+M+1
-        !        IE=(M*(2*NM-M)+1)/4*3+M*(2*NM-M+1)/2+M
         IE=(M*(2*NM-M)+1)/4*3+M*(2*NM-M+1)/2+M+1
         IJ=(M-1)*(2*NM-M)/16+M
         CALL LXSSWV(NM,NN,JM,JV,JR,M,S1(NS),S2(NS), &
@@ -75,34 +56,16 @@ SUBROUTINE SXTS2V(MM,NM,NN,IM,JM,S1,S2,G1,G2,IT,T,P,R,JC,W,IPOW)
              & R(IE),JC(IJ),WORK,WS,IPOW,0_8)
      END IF
   END DO
-!$omp end do
+  !$omp end do
+  CALL MXFREE(WP)
+  CALL MXFREE(WSP)
 !$omp end parallel
 
-  DO IP=0,NP-1
-     IF(LP(IP)) THEN
-        CALL MXFREE(WP(IP))
-        CALL MXFREE(WSP(IP))        
-     END IF
-  END DO
-
-  DEALLOCATE(WSP)  
-  DEALLOCATE(WP)
-  DEALLOCATE(LP)  
-
-  NP=MIN(JM/JV,NPMAX)
-  ALLOCATE(WP(0:NP-1))
-  ALLOCATE(LP(0:NP-1))
-  LP=.FALSE.
-  IP=0
-!$omp parallel private(IP,WORK) num_threads(NP)
-!$omp do schedule(dynamic)
+!$omp parallel private(WP,WORK) num_threads(NP)
+  CALL MXALLC(WP,JV*IM)
+  CALL C_F_POINTER(WP,WORK,[JV*IM])
+  !$omp do schedule(dynamic)
   DO JD=1,JM/JV
-!$      IP=omp_get_thread_num()
-     IF(.NOT.LP(IP)) THEN
-        CALL MXALLC(WP(IP),JV*IM)
-        CALL C_F_POINTER(WP(IP),WORK,[JV*IM])
-        LP(IP)=.TRUE.
-     END IF
      IF(JV.EQ.4) THEN     
         CALL SXQTB1(JM/JV,MM,IM,W(1+2*JV*(JD-1),1),WORK)
      ELSE IF(JV.EQ.8) THEN     
@@ -119,25 +82,15 @@ SUBROUTINE SXTS2V(MM,NM,NN,IM,JM,S1,S2,G1,G2,IT,T,P,R,JC,W,IPOW)
         CALL SXLTB2(JV,IM,WORK,G1(1+JV*IM*(JD-1)))
      END IF
   END DO
-!$omp end do
+  !$omp end do
+  CALL MXFREE(WP)    
 !$omp end parallel
-  DO IP=0,NP-1
-     IF(LP(IP)) THEN
-        CALL MXFREE(WP(IP))
-     END IF
-  END DO
 
-  LP=.FALSE.
-  IP=0
-!$omp parallel private(IP,WORK) num_threads(NP)
-!$omp do schedule(dynamic)
+!$omp parallel private(WP,WORK) num_threads(NP)
+  CALL MXALLC(WP,JV*IM)
+  CALL C_F_POINTER(WP,WORK,[JV*IM])
+  !$omp do schedule(dynamic)
   DO JD=1,JM/JV
-!$      IP=omp_get_thread_num()
-     IF(.NOT.LP(IP)) THEN
-        CALL MXALLC(WP(IP),JV*IM)
-        CALL C_F_POINTER(WP(IP),WORK,[JV*IM])
-        LP(IP)=.TRUE.
-     END IF
      IF(JV.EQ.4) THEN     
         CALL SXQTB1(JM/JV,MM,IM,W(1+2*JV*(JD-1),2),WORK)
      ELSE IF(JV.EQ.8) THEN     
@@ -154,16 +107,8 @@ SUBROUTINE SXTS2V(MM,NM,NN,IM,JM,S1,S2,G1,G2,IT,T,P,R,JC,W,IPOW)
         CALL SXLTB2(JV,IM,WORK,G2(1+JV*IM*(JD-1)))
      END IF
   END DO
-!$omp end do
+  !$omp end do
+  CALL MXFREE(WP)    
 !$omp end parallel
 
-  DO IP=0,NP-1
-     IF(LP(IP)) THEN
-        CALL MXFREE(WP(IP))
-     END IF
-  END DO
-  
-  DEALLOCATE(WP)
-  DEALLOCATE(LP)  
-  
 END SUBROUTINE SXTS2V
