@@ -1,6 +1,6 @@
 !***********************************************************************
 ! ISPACK FORTRAN SUBROUTINE LIBRARY FOR SCIENTIFIC COMPUTING
-! Copyright (C) 1998--2021 Keiichi Ishioka <ishioka@gfd-dennou.org>
+! Copyright (C) 1998--2022 Keiichi Ishioka <ishioka@gfd-dennou.org>
 !
 ! This library is free software; you can redistribute it and/or
 ! modify it under the terms of the GNU Lesser General Public
@@ -35,6 +35,12 @@ SUBROUTINE SYQRJV(JM,JV)
      ELSE
         JV=JM/2
      END IF
+  ELSE IF(ICPU.EQ.100) THEN ! SX-Aurora
+     IF(MOD(JM,512).EQ.0) THEN
+        JV=256
+     ELSE
+        JV=JM/2
+     END IF
   ELSE
      JV=JM/2
   END IF
